@@ -3,6 +3,6 @@
 sender_canonical_maps = tcp:127.0.0.1:10001
 sender_canonical_classes = envelope_sender
 recipient_canonical_maps = tcp:127.0.0.1:10002
-recipient_canonical_classes = envelope_recipient
+recipient_canonical_classes = envelope_recipient,header_recipient
 
 
