@@ -42,10 +42,6 @@
 #endif
 #include <syslog.h>
 
-#ifndef VERSION
-#define VERSION "1.6"
-#endif
-
 static char *self = NULL;
 
 static size_t bind_service (const char *listen_addr, const char *service, int family, int* socks, size_t max_socks)
@@ -126,9 +122,13 @@ static char* url_decode (char *buf, size_t len, const char *input)
   if (!input || !output || len == 0) return NULL;
 
   while (*input && --len) {
-    if (*input == '%' && is_hexdigit(input[1]) && is_hexdigit(input[2])) {
-      *output++ = hex2char(++input);
-      input += 2;
+    if (*input == '%') {
+      if (is_hexdigit(input[1]) && is_hexdigit(input[2])) {
+        *output++ = hex2char(++input);
+        input += 2;
+      } else {
+        return 0;
+      }
     } else {
       *output++ = *input++;
     }
@@ -168,11 +168,19 @@ static void handle_forward (srs_t *srs, FILE *fp, const char *address, const cha
     len = strlen(*excludes);
     if (len >= addrlen) continue;
     if (strcasecmp(*excludes, &address[addrlen - len]) == 0 && (**excludes == '.' || address[addrlen - len - 1] == '@')) {
-      fputs ("500 Domain excluded from SRS\n", fp);
+      syslog (LOG_MAIL | LOG_INFO, "srs_forward: <%s> not rewritten: Domain excluded by policy", address);
+      fputs ("500 Domain excluded py policy\n", fp);
       fflush (fp);
       return;
     }
   }
+  if (srs_reverse(srs, value, sizeof(value), address) == SRS_SUCCESS)
+  {
+    fprintf (fp, "500 Already rewritten\n");
+    fflush (fp);
+    syslog (LOG_MAIL | LOG_NOTICE, "srs_forward: <%s> not rewritten: Valid SRS address for <%s>", address, value);
+    return;
+  }
   result = srs_forward(srs, value, sizeof(value), address, domain);
   if (result == SRS_SUCCESS) {
     output = url_encode(outputbuf, sizeof(outputbuf), value);
@@ -229,6 +237,7 @@ static void show_help ()
     "   -u<user>       switch user id after port bind (default: none)\n"
     "   -t<seconds>    timeout for idle client connections (default: 1800)\n"
     "   -X<domain>     exclude additional domain from address rewriting\n"
+    "   -A             always rewrite addresses\n"
     "   -e             attempt to read above parameters from environment\n"
     "   -D             fork into background\n"
     "   -4             force IPv4 socket (default: any)\n"
@@ -245,7 +254,7 @@ typedef void(*handle_t)(srs_t*, FILE*, const char*, const char*, const char**);
 int main (int argc, char **argv)
 {
   int opt, timeout = 1800, family = AF_UNSPEC, hashlength = 0, hashmin = 0;
-  int daemonize = FALSE;
+  int daemonize = FALSE, always_rewrite = FALSE;
   char *listen_addr = NULL, *forward_service = NULL, *reverse_service = NULL,
        *user = NULL, *domain = NULL, *chroot_dir = NULL;
   char separator = '=';
@@ -268,7 +277,7 @@ int main (int argc, char **argv)
   tmp = strrchr(argv[0], '/');
   if (tmp) self = strdup(tmp + 1); else self = strdup(argv[0]);
 
-  while ((opt = getopt(argc, argv, "46d:a:l:f:r:s:n:N:u:t:p:c:X::Dhev")) != -1) {
+  while ((opt = getopt(argc, argv, "46d:a:l:f:r:s:n:N:u:t:p:c:X::ADhev")) != -1) {
     switch (opt) {
       case '?':
         return EXIT_FAILURE;
@@ -317,6 +326,9 @@ int main (int argc, char **argv)
       case 'D':
         daemonize = TRUE;
         break;
+      case 'A':
+        always_rewrite = TRUE;
+        break;
       case 'h':
         show_help();
         return EXIT_SUCCESS;
@@ -379,7 +391,7 @@ int main (int argc, char **argv)
         }
         break;
       case 'v':
-        fprintf (stdout, "%s\n", VERSION);
+        fprintf (stdout, "%s\n", POSTSRSD_VERSION);
         return EXIT_SUCCESS;
     }
   }
@@ -387,7 +399,7 @@ int main (int argc, char **argv)
     fprintf (stderr, "%s: extra argument on command line: %s\n", self, argv[optind]);
     return EXIT_FAILURE;
   }
-  if (domain == NULL) {
+  if (domain == NULL || *domain == 0) {
     fprintf (stderr, "%s: You must set a home domain (-d)\n", self);
     return EXIT_FAILURE;
   }
@@ -439,7 +451,7 @@ int main (int argc, char **argv)
   /* Force loading of timezone info (suggested by patrickdk77) */
   now = time(NULL);
   localtime (&now);
-  /* We also have to lookup the uid of the unprivileged user for the same reason. */
+  /* We also have to lookup the uid of the unprivileged user before the chroot. */
   if (user) {
     errno = 0;
     pwd = getpwnam(user);
@@ -464,6 +476,10 @@ int main (int argc, char **argv)
   }
   /* Finally, we revert to the unprivileged user */
   if (pwd) {
+    if (setgid(pwd->pw_gid) < 0) {
+      fprintf(stderr, "%s: Failed to switch group id: %s\n", self, strerror(errno));
+      return EXIT_FAILURE;
+    }
     if (setuid(pwd->pw_uid) < 0) {
       fprintf (stderr, "%s: Failed to switch user id: %s\n", self, strerror(errno));
       return EXIT_FAILURE;
@@ -490,6 +506,7 @@ int main (int argc, char **argv)
   }
   fclose (sf);
 
+  srs_set_alwaysrewrite (srs, always_rewrite);
   srs_set_separator (srs, separator);
   if (hashlength)
     srs_set_hashlength (srs, hashlength);
@@ -546,7 +563,11 @@ int main (int argc, char **argv)
               return EXIT_FAILURE;
             }
             key = url_decode(keybuf, sizeof(keybuf), token);
-            if (!key) break;
+            if (!key) {
+              fprintf (fp, "500 Invalid request\n");
+              fflush(fp);
+              return EXIT_FAILURE;
+            }
             handler[sc](srs, fp, key, domain, excludes);
             fflush (fp);
             if (poll(fds, 1, timeout * 1000) <= 0) break;
